Index: libavcodec/cabac.h
===================================================================
--- libavcodec/cabac.h	(revision 6685)
+++ libavcodec/cabac.h	(working copy)
@@ -25,6 +25,37 @@
  * Context Adaptive Binary Arithmetic Coder.
  */
 
+//#define DEBUG_CABAC
+// This writes each CABAC decision into the text file cabac.log.
+// Useful for debugging the order of syntax elements etc.
+// (in combination with the corresponding file of the encoder and
+// the diff tool).
+#ifdef DEBUG_CABAC
+#include <stdio.h>
+static FILE* cabac_log_file;
+static int cabac_log_last_ctx;
+static uint8_t *cabac_state_base;
+#ifdef fprintf
+#undef fprintf
+#endif
+#define cabac_log( ctx, v ) \
+    if( cabac_log_last_ctx == (ctx) ) \
+        fprintf( cabac_log_file, " %d", v ); \
+    else \
+        fprintf( cabac_log_file, "\n%03d: %d", (ctx), v ); \
+    cabac_log_last_ctx = (ctx);
+#define cabac_log_init() \
+    cabac_log_file = fopen( "cabac.log", "a" ); \
+    fprintf( cabac_log_file, "--" ); \
+    cabac_log_last_ctx = -2;
+#define cabac_log_end() \
+    fprintf( cabac_log_file, "\n" ); \
+    fclose( cabac_log_file );
+#else
+#define cabac_log( ctx, v )
+#define cabac_log_init()
+#define cabac_log_end()
+#endif
 
 //#undef NDEBUG
 #include <assert.h>
@@ -58,7 +89,6 @@
 void ff_init_cabac_decoder(CABACContext *c, const uint8_t *buf, int buf_size);
 void ff_init_cabac_states(CABACContext *c);
 
-
 static inline void put_cabac_bit(CABACContext *c, int b){
     put_bits(&c->pb, 1, b);
     for(;c->outstanding_count; c->outstanding_count--){
@@ -561,6 +591,7 @@
         refill2(c);
 #endif /* BRANCHLESS_CABAC_DECODER */
 #endif /* ARCH_X86 */
+    cabac_log( state - cabac_state_base, bit );
     return bit;
 }
 
@@ -579,9 +610,11 @@
         refill(c);
 
     if(c->low < c->range){
+        cabac_log( -1, 0 );
         return 0;
     }else{
         c->low -= c->range;
+        cabac_log( -1, 1 );
         return 1;
     }
 }
@@ -596,6 +629,7 @@
         renorm_cabac_decoder_once(c);
         return 0;
     }else{
+        cabac_log_end();
         return c->bytestream - c->bytestream_start;
     }
 }
Index: libavcodec/h264.c
===================================================================
--- libavcodec/h264.c	(revision 6685)
+++ libavcodec/h264.c	(working copy)
@@ -7395,6 +7395,11 @@
         ff_init_cabac_decoder( &h->cabac,
                                s->gb.buffer + get_bits_count(&s->gb)/8,
                                ( s->gb.size_in_bits - get_bits_count(&s->gb) + 7)/8);
+#ifdef DEBUG_CABAC
+        cabac_log_init();
+        cabac_state_base = &h->cabac_state;
+#endif
+
         /* calculate pre-state */
         for( i= 0; i < 460; i++ ) {
             int pre;
